@@ -18,8 +18,8 @@ import gzip
 
 # Functions ====================================================================
 
-def parse_gff_attributes(attr):
-    """Parse an entry from the "attr" column of a GFF3 file and yield it as
+def parse_gff_attributes(attr: str):
+    """Parse an entry from the "attr" column of a GFF3 file and return it as
     a dict
 
     Parameters
@@ -36,30 +36,60 @@ def parse_gff_attributes(attr):
     return dict(pair.split('=') for pair in attr.split(';'))
 
 
-def parse_gff(gff: str, type='gene'):
+def parse_gff(gff, type: str = 'gene', parse_attr=True):
     """Parse a GFF3 file and yield its lines as tuples
 
     Parameters
     ----------
-    gff : str
+    gff
         path to GFF3 file
     type
-        string indcicating feature type to include, or None to include all
+        string indicating feature type to include, or None to include all
         features
+    parse_attr : bool
+        if False, do not parse attributes
+
+    Yields
+    ------
+    seqid, start, end, strand, attr
+        coordinates of a feature
     """
 
-    with (gzip.open(gff, 'rt') if gff.endswith('.gz') else open(gff)) as f:
-        for l in f:
-             if not l.startswith('#'):
-                seqid, _, t, start, end, _, strand, _, attr = l.rstrip().split('\t')
+    with (gzip.open(gff, 'rt') if str(gff).endswith('.gz') else open(gff, 'r')) as f:
+        for line in f:
+             if not line.startswith('#'):
+                seqid, _, t, start, end, _, strand, _, attr = line.rstrip().split('\t')
                 if ((t == type) or (type is None)):
-                    yield (seqid, int(start), int(end), strand,
-                           parse_gff_attributes(attr))
+                    if parse_attr:
+                        yield (seqid, int(start), int(end), strand,
+                            parse_gff_attributes(attr))
+                    else:
+                        yield seqid, int(start), int(end), strand, '.'
 
 
-def generate_bed(gff, type='gene', attr_field='ID'):
+def generate_bed(gff, type: str = 'gene', tag: str = 'ID'):
+    """Convert rows of GFF3 data to BED data. This involves reordering the
+    columns to conform with BED format and shifting the coordinates to 0-based
+    half-open values.
+
+    Parameters
+    ----------
+    gff
+        path to GFF3 file
+    type
+        type of GFF3 record to parse
+    tag : str
+        GFF3 attribute tag to parse [ID]
+
+    Yields
+    ------
+    tuple
+        a row of BED data
+    """
+
     for seqid, start, end, strand, attr in parse_gff(gff, type=type):
-        yield seqid, start, end, attr[attr_field], 0, strand
+        yield seqid, start - 1, end, attr[tag], 0, strand
+
 
 def parse_arguments():
     parser = ArgumentParser(description='convert gff3 to bed12')
@@ -70,9 +100,9 @@ def parse_arguments():
 
 def main():
     args = parse_arguments()
-    transcripts = BedTool(tuple(generate_bed(args.gff, type='mRNA', attr_field='ID'))).sort()
-    exons = BedTool(tuple(generate_bed(args.gff, type='exon', attr_field='Parent'))).sort()
-    cds = BedTool(tuple(generate_bed(args.gff, type='CDS', attr_field='Parent'))).sort()
+    transcripts = BedTool(tuple(generate_bed(args.gff, type='mRNA', tag='ID'))).sort()
+    exons = BedTool(tuple(generate_bed(args.gff, type='exon', tag='Parent'))).sort()
+    cds = BedTool(tuple(generate_bed(args.gff, type='CDS', tag='Parent'))).sort()
     for t in transcripts:
         block_size, block_start = zip(
             *((str(exon.stop - exon.start), str(exon.start - t.start))
